@@ -5,21 +5,50 @@ import hcat.lib.functional
 from hcat.lib.functional import IntensityCellReject
 from hcat.backends.backend import Backend
 from hcat.models.r_unet import embed_model as RUnet
+from hcat.models.unext import UNeXT
 from hcat.train.transforms import median_filter, erosion
 import hcat.lib.utils
 
 from typing import Dict, Optional
 
+import matplotlib.pyplot as plt
+from torch.utils.tensorboard import SummaryWriter
+
+from torchvision.utils import draw_keypoints, flow_to_image, make_grid
+def write_progress(writer,  tag, epoch, images, probability_map, embedding, vector, centroids):
+    keypoint = centroids[0, :, [1, 0]].unsqueeze(0)
+
+    _a = images[0, [0, 0, 0], :, :, 15].cpu()
+    _b = probability_map[0, [0, 0, 0], :, :, 15].gt(0.5).float().cpu()
+
+
+    _overlay = hcat.lib.utils.make_embedding_image(embedding).unsqueeze(0)[[0,0,0],...]
+    _overlay = _overlay.div(20).clamp(0, 1)
+
+    _c = draw_keypoints(_overlay.mul(255).round().type(torch.uint8),
+                        keypoint, colors='red', radius=3).cpu()
+    _d = draw_keypoints(flow_to_image(vector[0, [1, 0], :, :, 15].float()),
+                        keypoint, colors='red', radius=3).cpu()
+
+    img_list = [_a, _b, _c, _d]
+    for i, im in enumerate(img_list):
+        assert isinstance(im, torch.Tensor), f'im {i} is not a Tensor instead it is a {type(im)}, {img_list[i]}'
+        assert img_list[0].shape == img_list[
+            i].shape, f'im {i} is has shape {im.shape}, not {img_list[0].shape}'
+
+    _img = make_grid(img_list, nrow=1, normalize=True, scale_each=True)
+
+    writer.add_image(tag, _img, epoch, dataformats='CWH')
 
 class SpatialEmbedding(Backend):
     def __init__(self,
-                 sigma: Optional[Tensor] = torch.tensor([0.02, 0.02, 0.02]),
+                 sigma: Optional[Tensor] = torch.tensor([2, 2, 2]),
                  device: Optional[str] = 'cuda',
                  model_loc: Optional[str] = None,
                  postprocessing: Optional[bool] = True,
                  scale: Optional[int] = 25,
                  figure: Optional[str] = None,
-                 architecture: Optional[RUnet] = RUnet):
+                 architecture: Optional[RUnet] = UNeXT):
         """
         Initialize Spatial embedding Algorithm.
 
@@ -41,6 +70,7 @@ class SpatialEmbedding(Backend):
         self.postprocessing = postprocessing
 
         self.figure = figure
+        self.writer = SummaryWriter()
 
         if self.url:
             self.model = self._model_loader_url(self.url, architecture, device)
@@ -50,10 +80,12 @@ class SpatialEmbedding(Backend):
         self.vector_to_embedding = torch.jit.script(
             hcat.lib.functional.VectorToEmbedding(scale=self.scale).requires_grad_(False).eval())
 
-        self.embedding_to_probability = torch.jit.script(
-            hcat.lib.functional.EmbeddingToProbability(scale=self.scale).requires_grad_(False).eval())
+        self.embedding_to_probability = hcat.lib.functional.EmbeddingToProbability(scale=self.scale).requires_grad_(False).eval()
 
-        self.estimate_centroids = hcat.lib.functional.EstimateCentroids(scale=self.scale).requires_grad_(False)
+        self.estimate_centroids = hcat.lib.functional.EstimateCentroids(scale=self.scale,
+                                                                        n_erode=2,
+                                                                        min_samples=80,
+                                                                        ).requires_grad_(False)
 
         self.filter = median_filter(kernel_targets=3, rate=1, device=device)
         self.binary_erosion = erosion(device=device)
@@ -66,6 +98,7 @@ class SpatialEmbedding(Backend):
         self.vec = None
         self.embed = None
         self.prob = None
+        self.e = 0
 
     def forward(self, image: Tensor) -> Tensor:
         """
@@ -94,7 +127,7 @@ class SpatialEmbedding(Backend):
         assert image.min() >= -1
         assert image.max() <= 1
 
-        # image = self.filter(image.to(self.device))
+        image = self.filter(image.to(self.device))
         image = image.to(self.device)
         b, c, x, y, z = image.shape
 
@@ -111,17 +144,20 @@ class SpatialEmbedding(Backend):
         self.prob = probability_map.cpu()
         self.vec = out.cpu()
 
-        out: Tensor = self.vector_to_embedding(out)
+        out: Tensor = self.vector_to_embedding(out, n=1)
         self.embed = out.cpu()
 
-        self.centroids: Dict[str, Tensor] = self.estimate_centroids(
-            self.vector_to_embedding(self.vec.cuda(), n=5), probability_map)
+        self.centroids: Dict[str, Tensor] = self.estimate_centroids(self.vector_to_embedding(self.vec.cuda(), n=10), probability_map)
 
         out: Tensor = self.embedding_to_probability(out, self.centroids, self.sigma)
 
-        # Reject cell masks that overlap or meet min Myo7a criteria
-        if self.postprocessing:
-            out: Tensor = self.intensity_rejection(out, image)
+        # write_progress(self.writer, 'test', self.e, image, probability_map.gt(0.5).float(), self.embed, vector=self.vec, centroids=self.centroids)
+
+        self.e += 1
+
+        # # Reject cell masks that overlap or meet min Myo7a criteria
+        # if self.postprocessing:
+        #     out: Tensor = self.intensity_rejection(out, image)
 
         if out.numel() == 0:
             return torch.zeros((b, 0, x, y, z), device=self.device)
